@@ -7,14 +7,15 @@ import cgi
 import deskbar
 import gnomedesktop
 import gnome.ui
-import gnomevfs
-import gobject
+import gio
+import glib
 import gtk
 import gtk.gdk
 import locale
 import logging
 import os
 import re
+import base64
 
 LOGGER = logging.getLogger(__name__)
 
@@ -95,7 +96,7 @@ def load_icon(icon, width=deskbar.ICON_HEIGHT, height=deskbar.ICON_HEIGHT):
     """
     pixbuf = None
     if icon != None and icon != "":
-        if icon.startswith("file://") and gnomevfs.exists(icon):
+        if icon.startswith("file://") and gio.File(uri=icon).query_exists():
             icon, flags = gnome.ui.icon_lookup(ICON_THEME, factory,
                 icon, "",
                 gnome.ui.ICON_LOOKUP_FLAGS_SHOW_SMALL_IMAGES_AS_THEMSELVES)
@@ -125,6 +126,33 @@ def load_icon(icon, width=deskbar.ICON_HEIGHT, height=deskbar.ICON_HEIGHT):
 def load_icon_from_icon_theme(iconname, size):
     return ICON_THEME.load_icon(iconname, size, gtk.ICON_LOOKUP_USE_BUILTIN)
 
+def load_base64_icon (base64_str):
+    """
+    Load a base64 encoded image as a C{gtk.gdk.Pixbuf}.
+    
+    @param base64_str: a C{string} with a base64 encoded image
+    @return: A C{gtk.gdk.Pixbuf} or a fallback icon in case there are errors
+        parsing C{base64_str}.
+    """
+    loader = gtk.gdk.PixbufLoader()
+    
+    try:
+        try:
+            loader.set_size(deskbar.ICON_HEIGHT, deskbar.ICON_HEIGHT)
+            loader.write(base64.b64decode(base64_str))
+        except Exception, e:
+            LOGGER.warning ("Failed to read base64 encoded image: %s" % e)
+        except gobject.GError, ee:
+            LOGGER.warning ("Failed to read base64 encoded image: %s" % ee)
+    finally:
+        loader.close()
+        
+    pixbuf = loader.get_pixbuf()
+    if pixbuf :
+        return pixbuf
+    
+    return _get_fall_back_icon()
+
 def _get_fall_back_icon():
     """
     @return: stock_unknown icon or C{None}
@@ -149,7 +177,7 @@ def is_executable(prog_path):
 
 def spawn_async(args):
     try:
-        gobject.spawn_async(args, flags=gobject.SPAWN_SEARCH_PATH)
+        glib.spawn_async(args, flags=glib.SPAWN_SEARCH_PATH)
         return True
     except Exception, e:
         message_dialog = gtk.MessageDialog(flags=gtk.DIALOG_MODAL, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_CLOSE)
@@ -168,30 +196,77 @@ def add_to_recent(uri):
         manager = gtk.recent_manager_get_default()
         manager.add_item(uri)
 
+def launch_default_for_uri(uri_string):
+    """
+    Open uri_string with the default application
+    according to content type
+    
+    @type uri_string: str 
+    """
+    gfile = gio.File(uri=uri_string)
+    appinfo = gfile.query_default_handler()
+    
+    if appinfo != None:
+        appinfo.launch([gfile], None)
+    else:
+        LOGGER.error("Could not detect default application for %s", gfile.get_uri())
+
+def launch_default_for_uri_and_scheme(uri_string):
+    """
+    Open uri_string with the default application
+    according to scheme
+    
+    @type uri_string: str 
+    """
+    gfile = gio.File(uri=uri_string)
+    appinfo = gfile.query_default_handler(None)
+    
+    if appinfo != None:
+        appinfo.launch_uris([uri_string], None)
+    else:
+        LOGGER.error("Could not detect default application for %s", gfile.get_uri())
+        
+def uri_has_default_handler(uri_string):
+    """
+    Returns True if there's a default application
+    to open the specified URI
+    @type uri_string: str 
+    """
+    gfile = gio.File(uri=uri_string)
+    try:
+        appinfo = gfile.query_default_handler(None)
+    except Exception, e:
+        LOGGER.error("Error retrieving default application for %s: %s",
+                     gfile.get_uri(), str(e))
+        return False
+    
+    return (appinfo != None)
+    
 def url_show_file(url, escape=True):
     """
     @param escape: Whether C{url} should be escaped or not 
     """
     try:
         if escape:
-            url = gnomevfs.escape_host_and_path_string(url)
-        gnomevfs.url_show(url)
+            url = gio.File(uri=url).get_uri()
+        launch_default_for_uri(url)
         add_to_recent(url)
     except Exception, e:
         executed = False
         try:
-            executed = spawn_async([gnomevfs.get_local_path_from_uri(url)])
+            executed = spawn_async([gio.File(uri=url).get_path()])
         except:
             if not executed:
                 if escape:
-                    url = gnomevfs.escape_host_and_path_string(dirname(url))
+                    url = gio.File(uri=url).get_uri()
                 url_show(url)
                 add_to_recent(url)
 
 def url_show(url):
     try:
-        gnomevfs.url_show(url)
+        launch_default_for_uri(url)
     except Exception, e:
+        LOGGER.exception(e)
         message_dialog = gtk.MessageDialog(flags=gtk.DIALOG_MODAL, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_CLOSE)
         message_dialog.set_markup("<span size='larger' weight='bold'>%s</span>\n\n '%s'" % (
             _("Cannot show URL:"), cgi.escape(url)))
@@ -200,6 +275,9 @@ def url_show(url):
         if resp == gtk.RESPONSE_CLOSE:
             message_dialog.destroy()
 
+# Make gtk.LinkButtons call url_show()
+gtk.link_button_set_uri_hook (lambda button, url : url_show(url))
+
 def get_proxy():
     # TODO: Very dirty, should use CoreImpl class
     deskbarapplet = GconfStore.get_instance()
